import unittest
from py2048.Board import *
from copy import deepcopy
import itertools as it
from measures import *

class TestBoard(unittest.TestCase):
    def general_test(self, tile_init_args : list, assertion_list : list, dir : Direction) -> None:
        """Tuple format : grid position (tuple), number
        tile_init_args should be a list of places where you want to put tiles. \n
        Each element of assertion list will be used to check the number at a specific grid position."""
        boards = [Board() for x in range(0, 3)]

        tile_adders = []
        for tup in tile_init_args:
            tile_adders.append(lambda x : x.group.add(Board.Tile(pyg.Vector2(tup[0]), tup[1])))

        tile_arg_perms = list(it.permutations(tile_init_args))

        for board, perm in zip(boards, tile_arg_perms): # Add the tiles in different orders
            board.group.empty()
            for arg in perm:
                board.group.add(Board.Tile(pyg.Vector2(arg[0]), arg[1]))
            board.move_board(dir)

        #Wait until the board has finished moving
        arrays_moving = True
        while arrays_moving:
            arrays_moving = False
            for board in boards:
                board.update()
                if board.array_moving:
                    arrays_moving = True
        
        for board in boards:
            board_list = board.generate_list()
            
            #Test if there board is in the expected configuration
            for a_grid, a_num in assertion_list:
                assert(board_list[a_grid[0]][a_grid[1]].number == a_num)
            
            #Test if there are tiles which occupy the same grid position
            tiles_in_list = 0
            for row in board_list:
                for tile in row:
                    if tile != 0:
                        tiles_in_list += 1

            assert(tiles_in_list == len(board.group))

    def test_vertical_upper(self):
        self.general_test([((0, 0), NUMBER_LIST[0]), ((0, 1), NUMBER_LIST[0]), ((0, 2), NUMBER_LIST[1])], \
            [((0, 0), NUMBER_LIST[1]), ((0, 1), NUMBER_LIST[1])], Direction.UP)
        print("Testing two 2 tiles in the first vertical column followed by a 4 below them, then moving up")

    def test_vertical_lower(self):
        self.general_test([((0, 0), NUMBER_LIST[0]), ((0, 1), NUMBER_LIST[0]), ((0, 2), NUMBER_LIST[1])], \
            [((0, BOARD_SIZE - 2), NUMBER_LIST[1]), ((0, BOARD_SIZE - 1), NUMBER_LIST[1])], Direction.DOWN)
        print("Testing two 2 tiles in the first vertical column followed by a 4 below them, then moving down")

    def test_horizontal_upper(self):
        self.general_test([((0, 0), NUMBER_LIST[0]), ((1, 0), NUMBER_LIST[0]), ((2, 0), NUMBER_LIST[1])], \
            [((0, 0), NUMBER_LIST[1]), ((1, 0), NUMBER_LIST[1])], Direction.LEFT)
        print("Testing two 2 tiles in the first row followed by a 4 in front of them, then moving left")

    def test_horizontal_lower(self):
        self.general_test([((0, 0), NUMBER_LIST[0]), ((1, 0), NUMBER_LIST[0]), ((2, 0), NUMBER_LIST[1])], \
            [((BOARD_SIZE - 2, 0), NUMBER_LIST[1]), ((BOARD_SIZE - 1, 0), NUMBER_LIST[1])], Direction.RIGHT)
        print("Testing two 2 tiles in the first row followed by a 4 in front of them, then moving right")

if __name__ == "__main__":
    unittest.main()


import sys

from py2048.downloader import download_assets
sys.path.append("../..")

import pygame as pyg
from pygame._sdl2.video import Window
#import py2048.move_window as mw
from py2048.measures import *
from py2048.Board import *
from py2048.downloader import *
import itertools as it

def main():
    pyg.init()
    Direction.init()

    if not os.path.exists(font_path):
        download_assets()

    Board.Tile.init()

    monitor = pyg.Vector2(pyg.display.Info().current_w, pyg.display.Info().current_h)
    
    screen = pyg.display.set_mode(WINDOW_SIZE)
    
    window = Window.from_display_module()
    window.position = ((monitor.x - WINDOW_SIZE[0]) / 2, (monitor.y - WINDOW_SIZE[1]) / 2)

    pyg.display.set_caption("2048")

    board_instance = Board()

    game_clock = pyg.time.Clock()

    pressed = False
    running = True
    start_pos = (0, 0)
    while running:
        game_clock.tick(200)

        for event in pyg.event.get():
            mouse_pos = pyg.mouse.get_pos()
            if event.type == pyg.QUIT or (event.type == pyg.KEYDOWN and pyg.key.get_pressed()[pyg.K_ESCAPE]):
                running = False
                break
            elif event.type == pyg.KEYDOWN:
                if pyg.key.get_pressed()[pyg.K_r]:
                    board_instance.reset(wait = False)
                board_instance.on_keydown()
            #mw.check_event(window, event)
            
        if pyg.key.get_pressed()[pyg.K_ESCAPE]:
            break
        
        screen.fill((187, 173, 160))
        
        for placeholder in it.chain.from_iterable(PLACEHOLDER_ARRAY):
            pyg.draw.rect(screen, (205, 193, 180), placeholder, border_radius = 1)
        board_instance.draw()
        
        board_instance.update()

        pyg.display.flip()
    
    board_instance.write_file()
    pyg.quit()

if __name__ == "__main__":
    main()



import requests
import os
import appdirs
from sys import platform

user_data = appdirs.user_data_dir()
base_path = os.path.join(user_data, "py2048")
font_path = os.path.join(base_path, "ClearSans-Bold.ttf")
save_path = os.path.join(base_path, "board.json")

def download_assets():
    try:
        print(f"Making game directory at {base_path}")
        os.mkdir(base_path)
    except:
        print("Game directory detected...")
    with open(font_path, 'wb') as file:
        print("Downloading font...")
        r = requests.get("https://raw.github.com/xXCoolinXx/2048/master/Assets/Fonts/ClearSans-Bold.ttf")
        file.write(r.content)


import pygame as pyg
from collections import defaultdict

STEP = 5

DIMENSION = 95

BOARD_SIZE = 4

WINDOW_LENGTH = BOARD_SIZE*(DIMENSION + STEP) + STEP

WINDOW_SIZE = (WINDOW_LENGTH, WINDOW_LENGTH)

POS_VALUES = [STEP + j*(DIMENSION + STEP) for j in range(0, BOARD_SIZE)]

PLACEHOLDER_ARRAY = [[pyg.Rect(x, y, DIMENSION, DIMENSION) for y in POS_VALUES] for x in POS_VALUES]

NUMBER_LIST = [2**x for x in range(1, 20)]

NUMBER_COLOR_DICT = defaultdict(lambda: (0, 0, 0))

NUMBER_COLOR_DICT[NUMBER_LIST[ 0]] = (238, 228, 218)
NUMBER_COLOR_DICT[NUMBER_LIST[ 1]] = (237, 224, 200)
NUMBER_COLOR_DICT[NUMBER_LIST[ 2]] = (242, 177, 121)
NUMBER_COLOR_DICT[NUMBER_LIST[ 3]] = (245, 149,  99)
NUMBER_COLOR_DICT[NUMBER_LIST[ 4]] = (246, 124,  95)
NUMBER_COLOR_DICT[NUMBER_LIST[ 5]] = (246,  94,  59)
NUMBER_COLOR_DICT[NUMBER_LIST[ 6]] = (237, 207, 114)
NUMBER_COLOR_DICT[NUMBER_LIST[ 7]] = (237, 204,  97)
NUMBER_COLOR_DICT[NUMBER_LIST[ 8]] = (237, 200,  80)
NUMBER_COLOR_DICT[NUMBER_LIST[ 9]] = (237, 198,  65)
NUMBER_COLOR_DICT[NUMBER_LIST[10]] = (237, 194,  46)


import pygame
from pygame._sdl2.video import Window

start_pos = pygame.Vector2(0, 0) #Initial mouse position
pressed = False #Flag that denotes when the mouse is being continuously pressed down

def move_window(window : Window, start_mouse_pos : pygame.Vector2, new_mouse_pos : pygame.Vector2) -> None:
    """Moves the window by the offset between start_mouse_pos and new_mouse_pos"""
    buffer_screen = pygame.Surface((window.size[0], window.size[1]))
    buffer_screen.blit(pygame.display.get_surface(), pygame.display.get_surface().get_rect())

    window_pos_Vec2 = pygame.Vector2(window.position)
    window.position = window_pos_Vec2 + new_mouse_pos - start_mouse_pos

    screen = pygame.display.set_mode((window.size[0], window.size[1]), pygame.NOFRAME)
    screen.blit(buffer_screen, buffer_screen.get_rect())
    pygame.display.flip()

def check_event(window : Window, event : pygame.event, move_area : pygame.Rect = pygame.Rect(-1, -1, 1, 1)) -> None:
    """Takes a window and event and updates the window position accordingly. \n
       move_area can be used to set what area of the screen can be clicked in order to move the window. \n
       move_area defaults to a dummy rect which is then internally changed to the full window."""
    global start_pos, pressed
    if move_area == pygame.Rect(-1, -1, 1, 1):
        move_area = pygame.Rect((0, 0), window.size)

    mouse_pos = pygame.Vector2(pygame.mouse.get_pos())
    if move_area.collidepoint(mouse_pos):
        if event.type == pygame.MOUSEBUTTONDOWN:
            pressed = True
            start_pos = mouse_pos
        elif event.type == pygame.MOUSEMOTION and pressed:
            move_window(window, start_pos, mouse_pos)
        elif event.type == pygame.MOUSEBUTTONUP: #Pygame sometimes fails to recognize this event, make the window "stuck" to the mouse
            pressed = False
            move_window(window, start_pos, mouse_pos)
    else:
        pressed = False


import pygame as pyg

class Button(object):
    ALL_CLICKS = 0
    FIRST_CLICK = 1
    RELEASE = 2

    def __init__(self, rect, normal_color, hover_color, when_active):
        self.rect = rect

        self.normal_color =  normal_color
        self.hover_color = hover_color
        self.current_color = normal_color
        self.active_on = when_active
        self.waiting = False

    def check_hover(self):
        if self.rect.collidepoint(pyg.mouse.get_pos()):
            self.current_color = self.hover_color
            return True
        else:
            self.current_color = self.normal_color

    def check_click(self):
        if self.check_hover() and pyg.mouse.get_pressed()[0]:
            if self.active_on == Button.ALL_CLICKS:
                return True
            elif self.active_on == Button.FIRST_CLICK and not self.waiting:
                self.waiting = True
                return True
            elif self.active_on == Button.RELEASE:
                self.waiting = True
                return False
        elif self.waiting: #Mouse not hovering and not pressed but waiting flag is set
            waiting = False
            if self.active_on == Button.RELEASE:
                return True

    def draw(self):
        return pyg.draw.rect(pyg.display.get_surface(), self.current_color, self.rect)



import setuptools

setuptools.setup(
    install_requires=[
        "pygame",
        "appdirs",
        "jsonpickle",
        "results",
        "requests"
    ],
    entry_points='''
        [console_scripts]
        2048=py2048.__main__:main
    ''',
)


from tkinter import font
import pygame as pyg
from random import *
from py2048.measures import *
from copy import deepcopy
import jsonpickle
import os
import threading
from math import log10
import itertools as it
from py2048.Button import Button
from py2048.downloader import *

pyg.font.init()

def do_twice(func):
    def wrapper(*args, **kwargs):
        func(*args, **kwargs)
        func(*args, **kwargs)
    return wrapper

class Direction():
    UP    = pyg.Vector2( 0, -1) 
    DOWN  = pyg.Vector2( 0,  1) 
    LEFT  = pyg.Vector2(-1,  0) 
    RIGHT = pyg.Vector2( 1,  0) 
    NONE  = pyg.Vector2( 0,  0)
    LIST  = [] #Order : Up, down, left, right
    
    @classmethod
    def init(cls):
        cls.LIST = [cls.UP, cls.DOWN, cls.LEFT, cls.RIGHT]

class Board():
    HIGHER_NUMBER_CHANCE = 0.10

    class Tile(pyg.sprite.Sprite):
        font_dict = {}
        SPEED = 10

        def __init__(self, location, number):
            pyg.sprite.Sprite.__init__(self)
            self.number = number
            self.grid_coords = location
            self.dir = Direction.NONE
            self.update_rectangle()
            self.update_text()
            self.set_color()
            self.merged = False #Flag to ensure that a tile does not merge twice during a single loop (unintended behavior)

        @classmethod
        def init(cls):
            if not cls.font_dict:
                cls.font_dict = {digit : pyg.font.Font(font_path, \
                    int(2 * DIMENSION / (digit + 1))) for digit in range(3, 7 + 1)}


        def update_rectangle(self):
            self.rect = PLACEHOLDER_ARRAY[int(self.grid_coords.x)][int(self.grid_coords.y)].copy()

        def update_text(self):
            self.text = Board.Tile.font_dict[max(int(log10(self.number)) + 1, 3)].render(str(self.number), True, (249, 246, 242) if self.number > NUMBER_LIST[1] else (119, 110, 101))

        def initiate_move(self, dir):
            self.dir = dir

        def update(self, tile_list):
            if self.rect.left not in POS_VALUES or self.rect.top not in POS_VALUES or self._can_move(tile_list):
                self.rect.move_ip(Board.Tile.SPEED*self.dir)
            else:
                self.dir = Direction.NONE

            self._update_grid_coords()

        def _update_grid_coords(self):
            if self.rect.left in POS_VALUES and self.rect.top in POS_VALUES:
                self.grid_coords = pyg.Vector2(POS_VALUES.index(self.rect.left), POS_VALUES.index(self.rect.top))

        def _can_move(self, tile_list):
            return self.can_move(self.dir, tile_list)

        def can_move(self, dir, tile_list):
            if dir != Direction.NONE and \
               0 <= self.grid_coords.x + dir.x <= BOARD_SIZE - 1 and 0 <= self.grid_coords.y + dir.y <= BOARD_SIZE - 1:
                next_coords = self.grid_coords + dir
                next = tile_list[int(next_coords.x)][int(next_coords.y)]
                if next == 0 or (next.number == self.number and not (self.merged or next.merged)) or next.can_move(dir, tile_list):
                    return True 
            return False

        def set_color(self):
            self.color = NUMBER_COLOR_DICT[self.number]

        def double(self):
            self.number = NUMBER_LIST[NUMBER_LIST.index(self.number) + 1]
            self.merge()
            self.set_color()
            self.update_text()

        def merge(self):
            self.merged = True
        
        def draw(self):
            screen = pyg.display.get_surface()
            text_rect = self.text.get_rect()
            text_rect.center = self.rect.center

            rect = pyg.draw.rect(screen, self.color, self.rect, border_radius = 1)
            screen.blit(self.text, text_rect)
            return rect

    def __init__(self):
        self.group = pyg.sprite.Group()
        self.array_moving = False
        self.try_read_file()

    @do_twice
    def start(self):
        board_list = self.generate_list()
        self.add_tile(board_list)

    def reset(self, wait = True):
        if wait:
            event = threading.Event()
            event.wait(timeout=2)
        self.group.empty()
        self.start()
    
    def add_tile(self, board_list):
        zeros = []
        for x, row in enumerate(board_list):
            for y, tile_space in enumerate(row):
                if tile_space == 0:
                    zeros.append(pyg.Vector2(x, y))
        if zeros:
            self.group.add(Board.Tile(choice(zeros), NUMBER_LIST[1] if random() <= Board.HIGHER_NUMBER_CHANCE else NUMBER_LIST[0]))

    def move_board(self, dir):
        for tile in self.group:
            tile.initiate_move(dir)

    def generate_list(self):
        a = [[0 for y in range(0, BOARD_SIZE)] for x in range(0, BOARD_SIZE)]
        for tile in self.group:
            a[int(tile.grid_coords.x)][int(tile.grid_coords.y)] = tile
        return a
        
    def draw(self):
        rect_list = []
        for tile in self.group:
            rect_list.append(tile.draw())
        pyg.display.update(rect_list)

    def update(self):
        board_list = self.generate_list()

        last_array_moving = deepcopy(self.array_moving)
        self.array_moving = False

        for tile in self.group:
            tile.update(board_list)
            if tile.dir != Direction.NONE:
                self.array_moving = True
        if last_array_moving:
            self.merge()
            if not self.array_moving:
                for tile in self.group:
                    tile.merged = False
                self.add_tile(board_list)
        if self.check_loss(board_list):
            self.reset()

    def on_keydown(self):
        if not self.array_moving:
            dir = Direction.NONE
            if pyg.key.get_pressed()[pyg.K_UP]:
                dir = Direction.UP
            elif pyg.key.get_pressed()[pyg.K_DOWN]:
                dir = Direction.DOWN
            elif pyg.key.get_pressed()[pyg.K_LEFT]:
                dir = Direction.LEFT
            elif pyg.key.get_pressed()[pyg.K_RIGHT]:
                dir = Direction.RIGHT
            if dir != Direction.NONE:
                self.move_board(dir)
    def merge(self):
        for a, b in it.combinations(self.group, 2):
            if a.rect.center == b.rect.center:
                    a.double()
                    b.kill()

    def check_loss(self, tile_list):
        if len(self.group) == BOARD_SIZE**2:
            for tile in self.group:
                if tile.can_move(Direction.UP, tile_list)   or tile.can_move(Direction.DOWN, tile_list) or \
                   tile.can_move(Direction.LEFT, tile_list) or tile.can_move(Direction.RIGHT, tile_list): 
                    #Determines if any box can move in any direction
                    return False
            return True

    def try_read_file(self):
        if os.path.isfile(save_path) and os.path.getsize(save_path) != 0: 
            with open(save_path, 'r') as f:
                tile_list = jsonpickle.decode(f.read())
                for tile in tile_list:
                    tile.add(self.group)
                    tile.update_text()
                    tile.update_rectangle()
                    tile.set_color()
                f.close()
        else:
            self.start()

    def write_file(self):
        with open(save_path, 'w') as f:
            tile_list = []
            for tile in self.group:
                tile.kill() #Makes sure that the group itself is not saved
                tile_list.append(tile)
            f.write(jsonpickle.encode(tile_list, indent=4))
            f.close()